@@ -13,7 +13,7 @@
 import datetime
 import uuid
 
-from keystoneclient import fixture
+from keystoneauth1 import fixture
 import mock
 import six
 import testtools
